"""
Pokemon Random Battle Data Package

A Python package providing offline access to Pokemon random battle data 
with automatic updates from the official source repository.
"""

__version__ = "0.1.15"

from .core import PokemonData, RandBatsData
from .formats import (
    get_pokemon, update_data, list_pokemon,
    get_smogon_sets, list_smogon_pokemon
)
from .smogon import SmogonSets

__all__ = [
    'PokemonData',
    'RandBatsData',
    'SmogonSets',
    'get_pokemon', 
    'update_data',
    'list_pokemon',
    'get_smogon_sets',
    'list_smogon_pokemon',
] 